
#include <stdio.h>
#include <mem.h>
#include <string.h>
#include <stdlib.h>

#include "wait.h"
#include "car.h"
#include "svga.h"
#include "pal.h"
#include "keyb.h"
#include "cgdk.h"
#include "timer.h"
#include "tm.h"

#include "plasma.h"
#include "plg000.h"

#define PLASMA_IMAGE_CAR_FILENAME CAR_EMULATION
#define PLASMA_IMAGE_FILENAME "plg000.cpi"

int main (void)
{
  TPLASMA_image *p;
  int phandle;
  int videoMode;

  printf ("\n"
          "PLASMA - example of the CGDK plasma engine\n"
          "\n%s\n",CGDK_copyright);

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  KEYB_init ();

  if (SVGA_compatible >= VGA_COMPATIBLE)
  {
    if ((_argc > 1) && ( (stricmp (_argv [1], "/l") == 0) || (stricmp (_argv [1], "-l") == 0) ) )
    {
      printf ("Loading ...\n");
      p = PLASMA_load (PLASMA_IMAGE_FILENAME, PLASMA_IMAGE_CAR_FILENAME);
      if (p == NULL)
      {
        printf ("Error loading plasma image.\n");
        return 1;
      }
    }
    else
    {
      printf ("Generating, please wait ...\n");
      p = PLASMA_g000 (1, WAIT_flashPrint);
      printf ("\n");
    }

    if (p == NULL)
    {
      printf ("Error creating plasma image.\n");
      return 2;
    }
    videoMode = SVGA_recommendVideoMode (p->videoSizeX, p->videoSizeY, 1, VMODE_LINEAR);
    if (videoMode)
    {
      SVGA_setVideoMode (videoMode);
      if ((SVGA_maxX < p->videoSizeX) || (SVGA_maxY < p->videoSizeY))
      {
        CGDK_exit ("No matching video mode available.\n", EXIT_FAILURE);
      }
      PLASMA_setVideoBuf (p, SVGA_linearBuf);

      while (KEYB_get () != KEY_ESC)
      {
        SVGA_wait (1);
        PLASMA_nextStep (p);
      }

      PAL_fadeAway (NULL, 50, 0, 256);
      PAL_fadeOut (NULL, 50, 0, 256);

      PLASMA_unload (p);
    }
    else
    {
      printf ("Not enough linear video memory.\n");
    }
  }
  else
  {
    printf ("This program requires a VGA compatible videocard.\n");
  }

  return 0;
}
