
#include <stdlib.h>
#include "svga.h"
#include "cgdk.h"

#include "vsb.h"
#include "lin.h"

char *VSB_buf;
char *VSB_sbuf;
int VSB_virtualX;
int VSB_virtualY;
long VSB_virtual;
int VSB_curX;
int VSB_curY;
int VSB_curXNext;
int VSB_curYNext;

void VSB_virtual2Real (int *_x, int *_y)
{
  int x = *_x;
  int y = *_y;

  if (x < 0)
  {
    x += VSB_virtualX;
    y --;
  }
  if (y < 0)
  {
    y += VSB_virtualY;
  }

  if (x >= VSB_virtualX)
  {
    x -= VSB_virtualX;
    y ++;
  }

  if (y >= VSB_virtualY)
  {
    y -= VSB_virtualY;
  }
  *_x = x;
  *_y = y;
}

void VSB_putPixel (int x, int y, int color)
{
  long offset;

  x += VSB_curX;
  y += VSB_curY;
  VSB_virtual2Real (&x, &y);

  offset = x + (unsigned)y*VSB_virtualX;
  if (offset < VSB_virtual)
  {
    ((char *) VSB_buf) [offset] = color;
  }
}


int VSB_getPixel (int x, int y)
{
  long offset;

  x += VSB_curX;
  y += VSB_curY;
  VSB_virtual2Real (&x, &y);

  offset = x + (unsigned)y*VSB_curX;
  if (offset < SVGA_virtual)
  {
    return VSB_buf [offset];
  }
  return 0;
}


void VSB_done (void)
{
  if (VSB_buf != NULL)
  {
    _free (VSB_buf);
    VSB_buf = NULL;
  }
  if (VSB_sbuf != NULL)
  {
    _free (VSB_sbuf);
    VSB_sbuf = NULL;
  }

}

int VSB_init (void)
{
  VSB_virtualX = SVGA_maxX+SVGA_maxXPlus;
  VSB_virtualY = SVGA_maxY+SVGA_maxYPlus;
  VSB_virtual = ((long)VSB_virtualX)*VSB_virtualY;

  VSB_curX = 0;
  VSB_curY = 0;

  VSB_done ();

  if (VSB_virtual)
  {
    #ifdef __TURBOC__
      if (VSB_virtual >= 0x10000)
      {
        return 3;
      }
    #endif
    VSB_buf = (char *) _alloc (VSB_virtual);
    if (VSB_buf == NULL)
    {
      return 1;
    }

    VSB_sbuf = (char *) _alloc (VSB_virtual);
    if (VSB_sbuf == NULL)
    {
      VSB_done ();
      return 2;
    }
  }
  return 0;
}



void VSB_getWin (char *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff, soff;
  int sizeX, sizeY;
  int restX, restY;
  int virtualX;

  X += VSB_curX;
  Y += VSB_curY;
  VSB_virtual2Real (&X, &Y);

  virtualX = VSB_virtualX;

  voff = X+Y*virtualX;

  sizeY = (VSB_virtual-voff)/virtualX;

  if (sizeY >= SIZE_Y)
  {
    LIN_movWin_ (win, formatX,
                 (char *)VSB_buf+voff, virtualX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      LIN_movWin_ (win, formatX,
                   (char *)VSB_buf+voff, virtualX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      soff=sizeY*formatX;
      voff+=sizeY*virtualX;

      restX = VSB_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      LIN_movWin_ ((char *)win+soff, sizeX, (char *)VSB_buf+voff, virtualX, sizeX, 1);
      LIN_movWin_ ((char *)win+soff+sizeX, SIZE_X-sizeX, VSB_buf, virtualX, SIZE_X-sizeX, 1);

      if (restY)
      {
        soff += formatX;
        voff = virtualX-restX;

        LIN_movWin_ (((char *)win)+soff, formatX,
                     (char *)VSB_buf+voff, virtualX, SIZE_X, restY);
      }
    }
  }
}

void VSB_putWin (char *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff, soff;
  int sizeX, sizeY;
  int restX, restY;
  int virtualX;

  X += VSB_curX;
  Y += VSB_curY;
  VSB_virtual2Real (&X, &Y);

  virtualX = VSB_virtualX;

  voff = X+Y*virtualX;

  sizeY = (VSB_virtual-voff)/virtualX;

  if (sizeY >= SIZE_Y)
  {
    LIN_movWin_ ((char *)VSB_buf+voff, virtualX,
                 win, formatX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      LIN_movWin_ ((char *)VSB_buf+voff, virtualX,
                   win, formatX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      soff=sizeY*formatX;
      voff+=sizeY*virtualX;

      restX = VSB_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      LIN_movWin_ ((char *)VSB_buf+voff, virtualX, (char *)win+soff, sizeX, sizeX, 1);
      LIN_movWin_ (VSB_buf, virtualX, (char *)win+soff+sizeX, SIZE_X-sizeX, SIZE_X-sizeX, 1);

      if (restY)
      {
        soff += formatX;
        voff = virtualX-restX;

        LIN_movWin_ ((char *)VSB_buf+voff, virtualX,
                     ((char *)win)+soff, formatX, SIZE_X, restY);
      }
    }
  }
}

void VSB_putSpr (char *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff, soff;
  int sizeX, sizeY;
  int restX, restY;
  int virtualX;

  X += VSB_curX;
  Y += VSB_curY;
  VSB_virtual2Real (&X, &Y);

  virtualX = VSB_virtualX;

  voff = X+Y*virtualX;

  sizeY = (VSB_virtual-voff)/virtualX;

  if (sizeY >= SIZE_Y)
  {
    LIN_movSpr_ ((char *)VSB_buf+voff, virtualX,
                 win, formatX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      LIN_movSpr_ ((char *)VSB_buf+voff, virtualX,
                   win, formatX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      soff=sizeY*formatX;
      voff+=sizeY*virtualX;

      restX = VSB_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      LIN_movSpr_ ((char *)VSB_buf+voff, virtualX, (char *)win+soff, sizeX, sizeX, 1);
      LIN_movSpr_ (VSB_buf, virtualX, (char *)win+soff+sizeX, SIZE_X-sizeX, SIZE_X-sizeX, 1);

      if (restY)
      {
        soff += formatX;
        voff = virtualX-restX;

        LIN_movSpr_ ((char *)VSB_buf+voff, virtualX,
                     ((char *)win)+soff, formatX, SIZE_X, restY);
      }
    }
  }
}

/*
  if (((long)(sizeY*VSB_virtualX) + di) >= svgaSize)
  {
    for (j = 0; j < sizeY; j++)
    {
      for (i = 0; i < sizeX; i++)
      {
        if ((di+i) >= svgaSize) // check overflow
        {
          di -= svgaSize;
        }
        raw [i] = VSB_buf [di+i];
      }
      di+=VSB_virtualX;
      raw+=formatX;
    }
  }
  else
  {
    LIN_movWin_ (raw, formatX, VSB_buf+di, VSB_virtualX, sizeX, sizeY);
  }
}
  */

void VSB_sput (char *raw, int x, int y,
              int sizeX, int sizeY, int formatX)
{
  char *buf=VSB_buf;
  VSB_buf=VSB_sbuf;
  VSB_putWin(raw,x,y,sizeX,sizeY,formatX);
  VSB_buf=buf;
}

void VSB_sget (char *raw, int x, int y,
              int sizeX, int sizeY, int formatX)
{
  char *buf=VSB_buf;
  VSB_buf=VSB_sbuf;
  VSB_getWin (raw,x,y,sizeX,sizeY,formatX);
  VSB_buf=buf;
}

/*
static void VSB_engine (char *raw, int x, int y,
              int sizeX, int sizeY, int formatX, char put)
{
  int i,j;
  unsigned di;
  unsigned register svgaSize = VSB_virtual;

  x += VSB_curX;
  y += VSB_curY;
  VSB_virtual2Real (&x, &y);

  // (1)
  di = (unsigned)((y)*VSB_virtualX)+(x);

  if (put)
  {
    for (j = 0; j < sizeY; j++)
    {
      for (i = 0; i < sizeX; i++)
      {
        if (di >= svgaSize) // check overflow
        {
          di -= svgaSize;
        }
      if (*raw)
        {
          VSB_buf [di] = *raw;
        }
        raw++;
        di++;
      }
      di+=VSB_virtualX-sizeX;
      raw+=formatX-sizeX;
    }
  }
  else
  {
    for (j = 0; j < sizeY; j++)
    {
      for (i = 0; i < sizeX; i++)
      {
        if (di >= svgaSize) // check overflow
        {
          di -= svgaSize;
        }
        *raw = VSB_buf [di];
        raw++;
        di++;
      }
      di+=VSB_virtualX-sizeX;
      raw+=formatX-sizeX;
    }
  }

  /*
  // (2)
  for (j = 0; j < sy-1; j++)
  {
    for (i = 0; i < rx; i++)
    {
      if (put)
      {
        c = raw [j*formatX+i+sx];
        VSB_buf [(j+dy+1)*VSB_virtualX+i] = c;
      }
      else
      {
        raw [j*formatX+i+sx] = VSB_buf [(j+dy+1)*VSB_virtualX+i];
      }
    }
  }

  // (3)
  for (j = 0; j < ry; j++)
  {
    for (i = 0; i < sx; i++)
    {
      if (put)
      {
        c = raw [(j+sy)*formatX*i];
        VSB_buf [(j*VSB_virtualX)+i+dx] = c;
      }
      else
      {
        raw [(j+sy)*formatX*i] = VSB_buf [(j*VSB_virtualX)+i+dx];
      }
    }
  }

  // (4)
  for (j = 0; j < ry+1; j++)
  {
    for (i = 0; i < rx; i++)
    {
      if (put)
      {
        c = raw [(j+sy)*formatX-rx+i];
        VSB_buf [j*VSB_virtualX+i] = c;
      }
      else
      {
        raw [(j+sy)*formatX-rx+i] = VSB_buf [j*VSB_virtualX+i];
      }
    }
  }
}
*/
/*
void VSB_putWin (char *raw, int x, int y,
              int sizeX, int sizeY, int formatX)
{
  int i,j;
  unsigned di;
  unsigned register svgaSize = VSB_virtual;

  x += VSB_curX;
  y += VSB_curY;
  VSB_virtual2Real (&x, &y);

  di = (unsigned)((y)*VSB_virtualX)+(x);

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      if ((di+i) >= svgaSize) // check overflow
      {
        di -= svgaSize;
      }
      VSB_buf [di+i] = raw [i];
    }
    di+=VSB_virtualX;
    raw+=formatX;
  }
}

void VSB_putSpr (char *raw, int x, int y,
              int sizeX, int sizeY, int formatX)
{
  int i,j;
  unsigned di;
  unsigned register svgaSize = VSB_virtual;

  x += VSB_curX;
  y += VSB_curY;
  VSB_virtual2Real (&x, &y);

  di = (unsigned)((y)*VSB_virtualX)+(x);

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      if ((di+i) >= svgaSize) // check overflow
      {
        di -= svgaSize;
      }
      if (raw[i])
      {
        VSB_buf [di+i] = raw[i];
      }
    }
    di+=VSB_virtualX;
    raw+=formatX;
  }
}
*/
